@@ -12,8 +12,9 @@
 #' @param term If \code{x} is a data.frame, the column in \code{x} which identifies a term. Defaults to the second column
 #' in \code{x}.
 #' @param ... further arguments passed on to the methods
-#' @return a data.table with columns document, term and the summed freq. If freq is not in the dataset,
-#' will assume that freq is 1 for each row in \code{x}.
+#' @return a data.table with columns doc_id, term, freq indicating how many times a term occurred in each document.
+#' If freq occurred in the input dataset the resulting data will have summed the freq. If freq is not in the dataset,
+#' will assume that freq is 1 for each row in the input dataset \code{x}.
 #' @export
 #' @examples 
 #' ##
@@ -99,13 +100,56 @@ document_term_frequencies.character <- function(x, document=paste("doc", seq_alo
 }
 
 
+#' @title Add Term Frequency, Inverse Document Frequency and Okapi BM25 statistics to the output of document_term_frequencies
+#' @description Term frequency Inverse Document Frequency (tfidf) is calculated as the multiplication of
+#' \itemize{
+#' \item Term Frequency (tf): how many times the word occurs in the document / how many words are in the document
+#' \item Inverse Document Frequency (idf): number of documents / number of documents where the term appears
+#' }
+#' The Okapi BM25 statistic is calculated as the multiplication of the inverse document frequency
+#' and the weighted term frequency as defined at \url{https://en.wikipedia.org/wiki/Okapi_BM25}.
+#' @param x a data.table as returned by \code{document_term_frequencies} containing the columns doc_id, term and freq.
+#' @param k parameter k1 of the Okapi BM25 ranking function as defined at \url{https://en.wikipedia.org/wiki/Okapi_BM25}. Defaults to 1.2.
+#' @param b parameter b of the Okapi BM25 ranking function as defined at \url{https://en.wikipedia.org/wiki/Okapi_BM25}. Defaults to 0.5.
+#' @return a data.table with columns doc_id, term, freq and added to that the computed statistics
+#' tf, idf, tfidf, tf_bm25 and bm25.
+#' @export
+#' @examples 
+#' data(brussels_reviews_anno)
+#' x <- document_term_frequencies(brussels_reviews_anno[, c("doc_id", "token")])
+#' x <- document_term_frequencies_statistics(x)
+#' head(x)
+document_term_frequencies_statistics <- function(x, k = 1.2, b = 0.75){
+  ## r cmd check happiness
+  doc_id <- term <- freq <- tf <- idf <- tf_idf <- tf_bm25 <- bm25 <- NULL
+
+  ##
+  ## Term Frequency Inverse Document Frequency
+  ##   - tf: how many times the word occurs in the document / how many words are in the document
+  ##   - idf: number of documents / number of documents where the term appears
+  ##
+  nr_docs <- length(unique(x$doc_id))
+  x <- x[, tf := freq / sum(freq), by = list(doc_id)]
+  x <- x[, idf := log(nr_docs / uniqueN(doc_id)), by = list(term)]
+  x <- x[, tf_idf := tf * idf, ]
+  
+  ##
+  ## Okapi BM25
+  ##   - See https://en.wikipedia.org/wiki/Okapi_BM25
+  ##
+  average_doc_length <- x[, list(nr_terms = sum(freq)), by = list(doc_id)]
+  average_doc_length <- mean(average_doc_length$nr_terms)
+  x <- x[, tf_bm25 := (freq * (k + 1)) / (freq + (k * (1 - b + b * (sum(freq) / average_doc_length)))), by = list(doc_id)]
+  x <- x[, bm25 := tf_bm25 * idf, ]
+  x
+}
 
 
 
 #' @title Create a document/term matrix from a data.frame with 1 row per document/term
 #' @description Create a document/term matrix from a data.frame with 1 row per document/term as returned
 #' by \code{\link{document_term_frequencies}}
-#' @param x a data.frame with columns document, term and freq indicating how many times a term occurred in that specific document.
+#' @param x a data.frame with columns doc_id, term and freq indicating how many times a term occurred in that specific document.
 #' This is what \code{\link{document_term_frequencies}} returns.
 #' @param vocabulary a character vector of terms which should be present in the document term matrix even if they did not occur in the \code{x}
 #' @param ... further arguments currently not used
@@ -413,10 +457,12 @@ dtm_cor <- function(x) {
 #' x <- brussels_reviews_anno
 #' 
 #' ## rbind
-#' dtm1 <- document_term_frequencies(x = subset(x, doc_id %in% c("10049756", "10284782")))
-#' dtm1 <- document_term_matrix(dtm1, document = "doc_id", term = c("token"))
-#' dtm2 <- document_term_frequencies(x = subset(x, doc_id %in% c("10789408", "12285061", "35509091")))
-#' dtm2 <- document_term_matrix(dtm2, document = "doc_id", term = c("token"))
+#' dtm1 <- document_term_frequencies(x = subset(x, doc_id %in% c("10049756", "10284782")),
+#'                                   document = "doc_id", term = "token")
+#' dtm1 <- document_term_matrix(dtm1)
+#' dtm2 <- document_term_frequencies(x = subset(x, doc_id %in% c("10789408", "12285061", "35509091")),
+#'                                   document = "doc_id", term = "token")
+#' dtm2 <- document_term_matrix(dtm2)
 #' m <- dtm_rbind(dtm1, dtm2)
 #' dim(m)
 #' 
@@ -476,4 +522,31 @@ dtm_rbind <- function(x, y){
     y <- methods::cbind2(y, addright)
   }
   rbind2(x[, r], y[, r])
+}
+
+
+#' @title Column sums and Row sums for document term matrices
+#' @description Column sums and Row sums for document term matrices
+#' @param dtm an object returned by \code{\link{document_term_matrix}}
+#' @return a vector of row/column sums with corresponding names
+#' @export
+#' @aliases dtm_colsums dtm_rowsums
+#' @examples 
+#' x <- data.frame(
+#'  doc_id = c(1, 1, 2, 3, 4), 
+#'  term = c("A", "C", "Z", "X", "G"), 
+#'  freq = c(1, 5, 7, 10, 0))
+#' dtm <- document_term_matrix(x)
+#' x <- dtm_colsums(dtm)
+#' x
+#' x <- dtm_rowsums(dtm)
+#' head(x)
+dtm_colsums <- function(dtm){
+  Matrix::colSums(dtm)
+}
+
+#' @export
+#' @rdname dtm_colsums
+dtm_rowsums <- function(dtm){
+  Matrix::rowSums(dtm)
 }
\ No newline at end of file
