@@ -68,6 +68,9 @@
 #'                      tagger = "none", parser = "none")
 #' as.data.frame(x)
 #' 
+#' ## Mark on encodings: if your data is not in UTF-8 encoding, make sure you convert it to UTF-8 
+#' ## This can be done using iconv as follows for example
+#' udpipe_annotate(ud_dutch, x = iconv('Ik drink melk bij mijn koffie.', to = "UTF-8"))
 #' 
 #' ## cleanup for CRAN only - you probably want to keep your model if you have downloaded it
 #' file.remove("dutch-lassysmall-ud-2.0-170801.udpipe")
@@ -126,6 +129,14 @@ as.data.frame.udpipe_connlu <- function(x, ...){
   read_connlu(x, is_udpipe_annotation = TRUE)
 }
 
+#' @export
+#' @rdname as.data.table.udpipe_connlu
+as.data.table.udpipe_connlu <- function(x, ...){
+  out <- as.data.frame(x)
+  out <- data.table::setDT(out)
+  out
+}
+
 
 read_connlu <- function(x, is_udpipe_annotation = FALSE){
   ## R CMD check happyness
